#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
This file contains API calls for all Plecost functions
"""

__version__ = "1.0.0"
__all__ = ["run", "find_versions", "PlecostOptions", "PlecostInvalidReportFormat"]

from sys import version_info

from .libs.reporters import *  # noqa
from .libs.data import PlecostOptions
from .libs.versions import find_versions  # noqa


# --------------------------------------------------------------------------
#
# Command line options
#
# --------------------------------------------------------------------------
def run(config):
    """
    Main function of libs:
    - Find WordPress versions
    - Find outdated plugins

    :param config: PlecostOptions option instance
    :type config: `PlecostOptions`

    :raises: PlecostTargetNotAvailable, PlecostNotWordPressFound, PlecostWordListNotFound
    """

    # --------------------------------------------------------------------------
    # Checks Python version
    # --------------------------------------------------------------------------
    if version_info < (3, 3):
        raise RuntimeError("You need Python 3.3.x or higher to run Plecost")

    # Check reporter
    if config.report_filename is not None:
        # Select appropriate report.
        reporter_function = get_reporter(config.report_filename)

    # Find wordpress and plugins versions
    data = find_versions(config)

    # Generate reports
    if config.report_filename is not None:

        report = reporter_function(config.report_filename)

        # Get report content
        content = report.generate(data)

        # Save report content
        report.save(content)